from qtcurate.config import BASE_URL


class Qt:
    api_key = None
    url = None

    @staticmethod
    def init(key: str, environment: str = "") -> None:
        Qt.api_key = key
        if environment != "":
            Qt.url = f"http://{environment}.{BASE_URL}"
        else:
            Qt.url = f"https://{BASE_URL}"



import requests
from qtcurate.exceptions import QtConnectionError, QtRestApiError, QtArgumentError


def connect(method: str, uri: str, headers: str, data_type: str = None, data=None) -> requests.Response:
    s = requests.Session()
    res = None
    if data_type not in [None, "data", "files", "params"]:
        raise QtArgumentError("Unknown data type. params, data and files are allowed")
    if method.lower() == 'get':
        if data_type is None:
            try:
                res = s.get(uri, headers=headers)
            except requests.exceptions.RequestException as e:
                raise QtConnectionError(f"Connection error: {e}")
            if res.status_code not in [200, 201, 202, 204]:
                raise QtRestApiError(f"HTTP error: Full authentication is required to access this resource. "
                                     f"HTTP status code: {res.status_code}. Server message: {res.json()}")
        elif data_type == "params":
            try:
                res = s.get(uri, headers=headers, params=data)
            except requests.exceptions.RequestException as e:
                raise QtConnectionError(f"Connection error: {e}")
            if res.status_code not in [200, 201, 202, 204]:
                raise QtRestApiError(f"HTTP error: Full authentication is required to access this resource. "
                                     f"HTTP status code: {res.status_code}. Server message: {res.json()}")
    elif method.lower() == "delete":
        try:
            res = s.delete(uri, headers=headers)
        except requests.exceptions.RequestException as e:
            raise QtConnectionError(f"Connection error: {e}")
        if res.status_code not in [200, 201, 202, 204]:
            raise QtRestApiError(f"HTTP error: Full authentication is required to access this resource. "
                                 f"HTTP status code: {res.status_code}. Server message: {res.json()}")
    elif method.lower() == "post":
        if data_type == "data":
            try:
                res = s.post(uri, headers=headers, data=data)
            except requests.exceptions.RequestException as e:
                raise QtConnectionError(f"Connection error: {e}")
            if res.status_code not in [200, 201, 202, 204]:
                raise QtRestApiError(f"HTTP error: Full authentication is required to access this resource. "
                                     f"HTTP status code: {res.status_code}. Server message: {res.json()}")
        elif data_type == "files":
            try:
                res = s.post(uri, headers=headers, files=data)
            except requests.exceptions.RequestException as e:
                raise QtConnectionError(f"Connection error: {e}")
            if res.status_code not in [200, 201, 202, 204]:
                raise QtRestApiError(f"HTTP error: Full authentication is required to access this resource. "
                                     f"HTTP status code: {res.status_code}. Server message: {res.json()}")

    elif method.lower() == "put":
        if data_type == "data":
            try:
                res = s.put(uri, headers=headers, data=data)
            except requests.exceptions.RequestException as e:
                raise QtConnectionError(f"Connection error: {e}")
            if res.status_code not in [200, 201, 202, 204]:
                raise QtRestApiError(f"HTTP error: Full authentication is required to access this resource. "
                                     f"HTTP status code: {res.status_code}. Server message: {res.json()}")
        elif data_type == "files":
            try:
                res = s.put(uri, headers=headers, files=data)
            except requests.exceptions.RequestException as e:
                raise QtConnectionError(f"Connection error: {e}")
            if res.status_code not in [200, 201, 202, 204]:
                raise QtRestApiError(f"HTTP error: Full authentication is required to access this resource. "
                                     f"HTTP status code: {res.status_code}. Server message: {res.json()}")
        else:
            raise QtArgumentError("Unknown data type. data and files are allowed")
    return res



from __future__ import annotations
import json
import os
from typing import List, Optional
from qtcurate.exceptions import QtArgumentError, QtFileTypeError
from qtcurate.qt import Qt
from qtcurate.connect import connect


NoneType = type(None)
category = "category"
vocab_name = "dict_name"
vocab_id = "dict_id"
type_field = "type"
str_field = "str"
double_value = "doubleValue"
int_value = "intValue"
line = "line"
start = "start"
end = "end"


class Position:
    def __init__(self):
        self.start = None
        self.end = None
        self.line = None

    def set_start(self, pos_start: Optional[int]) -> Position:
        """ Set start position"""

        if isinstance(pos_start, (int, NoneType)):
            self.start = pos_start
        else:
            raise QtArgumentError("Argument type error: Int is expected as pos_start")
        return self

    def get_start(self) -> Optional[int]:
        """Get start position"""

        return self.start

    def set_end(self, pos_end: Optional[int]) -> Position:
        """ Set end position"""

        if isinstance(pos_end, (int, NoneType)):
            self.end = pos_end
        else:
            raise QtArgumentError("Argument type error: Int is expected as pos_end")
        return self

    def get_end(self) -> Optional[int]:
        """Get end position"""

        return self.end

    def set_line(self, pos_line: Optional[int]) -> Position:
        """Set line position"""

        if isinstance(pos_line, (int, NoneType)):
            self.line = pos_line
        else:
            raise QtArgumentError("Argument type error: Int is expected as pos_line")
        return self

    def get_line(self) -> Optional[int]:
        """Get line position"""

        return self.line


class FieldValue:
    def __init__(self):
        self.str_value = None

    def __repr__(self):
        return f"{self.str_value}"

    def set_str(self, str_val: Optional[str]) -> FieldValue:
        """Set str in FieldValue"""

        if isinstance(str_val, (str, NoneType)):
            self.str_value = str_val
        else:
            raise QtArgumentError("Argument type error: String is expected as str_val")
        return self

    def get_str(self) -> Optional[str]:
        """Get str in FieldValue"""

        return self.str_value


class Field:

    def __init__(self):
        self.vocab_name = None
        self.vocab_id = None
        self.category = None
        self.type = None
        self.str = None
        self.values = None
        self.position = None

    def __repr__(self):
        return f"{self.vocab_id}, {self.str}"

    def set_values(self, val: Optional[List]) -> Field:
        """Set values in Field"""

        if isinstance(val, (list, NoneType)):
            self.values = val
        else:
            raise QtArgumentError("Argument type error: List is expected as val")
        return self

    def get_values(self) -> Optional[List]:
        """Set values in Field"""

        return self.values

    def set_str(self, str_value: Optional[str]) -> Field:
        """Set str in Field"""

        if isinstance(str_value, (str, NoneType)):
            self.str = str_value
        else:
            raise QtArgumentError("Argument type error: String is expected as str_val")
        return self

    def get_str(self) -> Optional[str]:
        """Get str in Field"""

        return self.str

    def set_type(self, type_value: Optional[str]) -> Field:
        """Set type in Field"""

        if isinstance(type_value, (str, NoneType)):
            self.type = type_value
        else:
            raise QtArgumentError("Argument type error: String is expected as type_value")
        return self

    def get_type(self) -> Optional[str]:
        """Get type in Field"""

        return self.type

    def set_vocab_name(self, name: Optional[str]) -> Field:
        """Set vocabulary name"""

        if isinstance(name, (str, NoneType)):
            self.vocab_name = name
        else:
            raise QtArgumentError("Argument type error: String is expected as name")
        return self

    def get_vocab_name(self) -> Optional[str]:
        """Get vocabulary name"""

        return self.vocab_name

    def set_category(self, cat: Optional[str]) -> Field:
        """Set category"""

        if isinstance(cat, (str, NoneType)):
            self.category = cat
        else:
            raise QtArgumentError("Argument type error: String is expected as cat")
        return self

    def get_category(self) -> Optional[str]:
        """Get category"""

        return self.category

    def set_vocab_id(self, voc_id: Optional[str]) -> Field:
        """Set vocabulary id"""

        if isinstance(voc_id, (str, NoneType)):
            self.vocab_id = voc_id
        else:
            raise QtArgumentError("Argument type error: String is expected as voc_id")
        return self

    def get_vocab_id(self) -> Optional[str]:
        """Get vocabulary id"""

        return self.vocab_id

    def set_position(self, position: Optional[Position]) -> Field:
        """Set position"""

        if isinstance(position, (Position, NoneType)):
            self.position = position
        else:
            raise QtArgumentError("Argument type error: Position is expected as position")
        return self

    def get_position(self) -> Optional[Position]:
        """Get position"""

        return self.position


class Result:

    def __init__(self, dp_id):
        self.id = dp_id
        self.headers = {"X-API-Key": Qt.api_key}
        self.url = Qt.url

    def raw_exporter(self, path: str) -> None:
        """Exporting in JSON format"""

        if not isinstance(path, str):
            raise QtArgumentError("Argument type error: String is expected as path")
        directory = os.path.dirname(path)
        if len(directory) == 0:
            directory = "."
        if not (os.access(directory, os.W_OK) and os.access(directory, os.X_OK)):
            raise QtArgumentError("Argument error: No write permission")
        extension = os.path.splitext(path)[1].lower()
        if extension != ".json":
            raise QtFileTypeError("File type error: Please use json extension saving file")
        res = connect("get", f"{self.url}reports/{self.id}/json", self.headers)
        with open(path, "w") as json_file:
            json.dump(res.json(), json_file)

    def result_xlsx_exporter(self, path: str) -> None:
        """Exporting in Excel format"""

        if not isinstance(path, str):
            raise QtArgumentError("Argument type error: String is expected as path")
        directory = os.path.dirname(path)
        if len(directory) == 0:
            directory = "."
        if not (os.access(directory, os.W_OK) and os.access(directory, os.X_OK)):
            raise QtArgumentError("Argument error: No write permission")
        extension = os.path.splitext(path)[1].lower()
        if extension != ".xlsx":
            raise QtFileTypeError("File type error: Please use xlsx extension saving file")
        res = connect("get", f"{self.url}reports/{self.id}/xlsx", self.headers)

        with open(path, 'wb') as excel_file:
            excel_file.write(res.content)

    def read(self) -> List:
        """Convert to easy readable list"""

        result_list = []
        res = connect("get", f"{self.url}reports/{self.id}/json", self.headers)
        for item in res.json():
            if "values" in item and item["values"]:
                for i in item["values"]:
                    field = Field()
                    position = Position()
                    if start in i:
                        position.set_start(i[start])
                    if end in i:
                        position.set_end(i[end])
                    if line in i:
                        position.set_line(i[line])
                    field.set_position(position)
                    if str_field in i:
                        field.set_str(i[str_field])
                    if type_field in i:
                        field.set_type(i[type_field])
                    if category in i:
                        field.set_category(i[category])
                    if vocab_name in i:
                        field.set_vocab_name(i[vocab_name])
                    if vocab_id in i:
                        field.set_vocab_id(i[vocab_id])
                    list_field_ext = []
                    if "extIntervalSimples" in i and i["extIntervalSimples"]:
                        for ext_int_item in i["extIntervalSimples"]:
                            field_value = FieldValue()
                            if str_field in ext_int_item:
                                field_value.set_str(ext_int_item[str_field])
                            list_field_ext.append(field_value)
                    field.set_values(list_field_ext)
                    result_list.append(field)
        return result_list



from __future__ import annotations
from qtcurate.qt import Qt
import json
import os.path
from typing import Dict, Union, List
from qtcurate.exceptions import QtArgumentError, QtVocabularyError
from qtcurate.connect import connect

vocabulary_entries = "entries"
vocabulary_name = "name"


class Vocabulary(Qt):
    def __init__(self):
        self.headers = {"X-API-Key": Qt.api_key}
        self.temp_vocabulary = dict()
        self.temp_vocabulary[vocabulary_entries] = []
        self.temp_vocabulary[vocabulary_name] = None
        self.id = None
        self.input_stream = None
        self.url = f"{Qt.url}dictionaries/"

    def __repr__(self):
        return str({'id': self.id, 'name': self.temp_vocabulary[vocabulary_name],
                    'entries': self.temp_vocabulary[vocabulary_entries]})

    def get_id(self) -> str:
        """Get vocabulary id"""

        return str(self.id)

    def set_id(self, voc_id: str) -> Vocabulary:
        """Set vocabulary id"""

        if isinstance(voc_id, str):
            self.id = voc_id
        else:
            raise QtArgumentError("Argument type error: String is expected as id")
        return self

    def name(self, voc_name: str) -> Vocabulary:
        """Create a new name for vocabulary"""
        
        if isinstance(voc_name, str):
            self.temp_vocabulary[vocabulary_name] = voc_name
        else:
            raise QtArgumentError("Argument type error: String is expected as name")
        return self

    def get_name(self) -> str:
        """Get vocabulary name"""

        return self.temp_vocabulary[vocabulary_name]

    def entries(self, entry: Dict) -> Vocabulary:
        """Create vocabulary data"""
        
        if not isinstance(entry, Dict):
            raise QtArgumentError("Argument type error: Dictionary is expected as entry")
        elif len(entry) > 2 or len(entry) == 0:
            raise QtArgumentError("Argument error: Vocabulary must have 1 or 2 elements where keys are 'str' and "
                                  "optional 'category'."
                                  "Example {'str': 'some str', 'category': 'some category'} or just {'str':'some str'}")
        elif "str" not in entry.keys():
            raise QtArgumentError("Argument error: Vocabulary must have 1 or 2 elements where keys are 'str' and "
                                  "optional 'category'."
                                  "Example {'str': 'some str', 'category': 'some category'} or just {'str':'some str'}")
        elif len(set(entry.keys()).difference({"str", "category"})) != 0:
            raise QtArgumentError("Argument error: Vocabulary must have 1 or 2 elements where keys are 'str' and "
                                  "optional 'category'."
                                  "Example {'str': 'some str', 'category': 'some category'} or just {'str':'some str'}")
        else:
            self.temp_vocabulary[vocabulary_entries].append(entry)
        return self

    def clear(self) -> None:
        """Remove all temporary data"""

        self.temp_vocabulary[vocabulary_entries] = []
        self.temp_vocabulary[vocabulary_name] = None
        self.id = None
        self.input_stream = None

    def add_entry(self, str_key: Union[str, int, float], category: Union[str, int, float] = None) -> Vocabulary:
        """Create vocabulary data"""
        
        has_category = False
        if category is not None:
            has_category = True
        if not isinstance(str_key, (str, int, float)):
            raise QtArgumentError("Argument type error: String, integer or float are expected as key")
        elif not isinstance(category, (str, int, float)) and has_category:
            raise QtArgumentError("Argument type error: String, integer or float is expected as value")
        else:
            if has_category:
                self.temp_vocabulary[vocabulary_entries].append({'str': str_key, 'category': category})
            else:
                self.temp_vocabulary[vocabulary_entries].append({'str': str_key})
        return self

    def get_entries(self) -> List:
        """Get entries"""

        return self.temp_vocabulary[vocabulary_entries]

    def read(self) -> Dict:
        """List all vocabularies"""

        res = connect("get", self.url, self.headers)
        return res.json()

    def fetch(self, vocabulary_id: str) -> Vocabulary:
        """Fetch vocabulary by ID"""

        if not isinstance(vocabulary_id, str):
            raise QtArgumentError("Argument type error: String is expected as qt_id")

        res = connect("get", f"{self.url}{vocabulary_id}", self.headers)
        result = res.json()
        self.temp_vocabulary[vocabulary_name] = result[vocabulary_name]
        self.id = result['id']
        self.temp_vocabulary[vocabulary_entries] = result[vocabulary_entries]
        return self

    def delete(self, vocabulary_id: str) -> bool:
        """Delete existing vocabulary"""

        if not isinstance(vocabulary_id, str):
            raise QtArgumentError("Argument type error: String is expected as qt_id")
        res = connect("delete", f"{self.url}{vocabulary_id}", self.headers)

        return res.ok

    def source(self, file: str) -> Vocabulary:
        """Set source"""

        if not isinstance(file, str):
            raise QtArgumentError("Argument type error: String is expected as file")
        extension = os.path.splitext(file)[1].lower()
        if extension != ".tsv":
            raise QtArgumentError("Argument type error: TSV file expected")
        if not os.path.exists(file):
            raise QtArgumentError(f"Argument error: File {file} does not exist")
        self.input_stream = file
        return self

    def create(self) -> Vocabulary:
        """Create vocabulary data"""

        if self.temp_vocabulary[vocabulary_name] is None:
            raise QtVocabularyError("Vocab error: Please add name using name function")

        if self.input_stream is None:
            self.headers['Content-Type'] = 'application/json'
            if not self.temp_vocabulary[vocabulary_entries]:
                raise QtVocabularyError("Vocabulary error: Please add vocabulary using add_entry function")
            data = {'name': self.temp_vocabulary[vocabulary_name], 'entries': self.temp_vocabulary[vocabulary_entries]}
            res = connect("post", self.url, self.headers, "data", json.dumps(data))
            self.id = res.json()["id"]
            return self
        else:
            files = {
                'name': (None, self.temp_vocabulary[vocabulary_name]),
                'file': open(self.input_stream, 'rb')
            }
            res = connect("post", f"{self.url}upload", self.headers, "files", files)
            result = res.json()
            self.id = result["id"]
            self.temp_vocabulary[vocabulary_entries] = result[vocabulary_entries]
            return self

    def update(self, qt_id: str) -> Vocabulary:
        """Update existing vocabulary"""
        
        self.headers['Content-Type'] = 'application/json'
        if not isinstance(qt_id, str):
            raise QtArgumentError("Argument type error: String is expected as qt_id")
        if self.temp_vocabulary[vocabulary_name] is None:
            raise QtVocabularyError("Vocabulary error: Please add name using name function")
        if len(self.temp_vocabulary[vocabulary_entries]) == 0:
            raise QtVocabularyError("Vocabulary error: Please add vocabulary using add_entry function")
        data = {'name': self.temp_vocabulary[vocabulary_name], 'entries': self.temp_vocabulary[vocabulary_entries]}
        res = connect("put", f"{self.url}{qt_id}", self.headers, "data", json.dumps(data))
        self.temp_vocabulary[vocabulary_entries] = res.json()[vocabulary_entries]
        del self.headers['Content-Type']
        return self



from __future__ import annotations
from qtcurate.document import Document
from qtcurate.extractor import Extractor, Mode, ChunkMode, SearchMode, AnalyzeMode
from qtcurate.connect import connect
from time import sleep
import re
import json
from typing import Dict, List
from qtcurate.qt import Qt
from qtcurate.exceptions import QtArgumentError, QtModelError


tag_files = "files"
chunk = "chunk"
num_workers = "numWorkers"
description = "title"
vocab_id = "vocabId"
vocab_value_type = "vocabValueType"
validator = "phraseMatchingPattern"
phrase_groups = "phraseMatchingGroups"
exclude_utt = "excludeUttWithoutEntities"
search_vocabulary = "searchDictionaries"
between_values = "skipPatternBetweenValues"
search_mod = "searchMode"
analyze_strategy = "analyzeStrategy"
stop_word_list = "stopwordList"
synonym_l = "synonymList"
data_type = "dataType"
language = "language"


class Model:
    def __init__(self):
        self.headers = {"X-API-Key": Qt.api_key}
        self.temp_dictionary = dict()
        self.temp_dictionary[description] = None
        self.temp_dictionary[exclude_utt] = True
        self.temp_dictionary[num_workers] = 8
        self.temp_dictionary[chunk] = ChunkMode.PAGE.value
        self.temp_dictionary[search_vocabulary] = []
        self.index = None
        self.uuid = None
        self.id = None
        self.url = Qt.url

    def __repr__(self):
        return f"{self.id}, {self.temp_dictionary}"

    def get_id(self) -> str:
        """Get model id"""

        return str(self.id)

    def set_id(self, model_id: str) -> Model:
        """Set model id"""

        if isinstance(model_id, str):
            self.id = model_id
        else:
            raise QtArgumentError("Argument type error: String is expected as model_id")
        return self

    def set_chunk(self, value: ChunkMode) -> Model:
        """Set chunk mode"""

        if isinstance(value, ChunkMode):
            self.temp_dictionary[chunk] = value.value
        else:
            raise QtArgumentError("Argument type error: ChunkMode is expected as value")
        return self

    def get_uuid(self) -> str:
        """Get uuid"""

        return str(self.uuid)

    def set_description(self, title: str) -> Model:
        """Create title for model"""

        if isinstance(title, str):
            self.temp_dictionary[description] = title
        else:
            raise QtArgumentError("Argument type error: String is expected as description")
        return self

    def exclude_utt_without_entities(self, value: bool) -> Model:
        """Set exclude utt for mining data, this is optional parameter"""

        if isinstance(value, bool):
            self.temp_dictionary[exclude_utt] = value
        else:
            raise QtArgumentError("Argument type error: Boolean is expected as exclude utt without entities")
        return self

    def set_workers(self, value: int) -> Model:
        """Set workers"""

        if isinstance(value, int):
            self.temp_dictionary[num_workers] = value
        else:
            raise QtArgumentError("Argument type error: Integer is expected as num_workers")
        return self

    def with_documents(self, list_of_files: List) -> Model:
        """Create a list of existing files"""

        if isinstance(list_of_files, list):
            self.temp_dictionary[tag_files] = list_of_files
        else:
            raise QtArgumentError("Argument type error: Expected list of file IDs")
        return self

    def add_extractor(self, extractor: Extractor) -> Model:
        """Prepare dictionary for searching"""

        vocab_dict = dict()
        if extractor.get_vocab_id() is not None:
            vocab_dict[vocab_id] = extractor.get_vocab_id()
        if extractor.get_vocab_value_type() is not None:
            vocab_dict[vocab_value_type] = extractor.get_vocab_value_type()
        if extractor.get_type() is not None:
            vocab_dict[data_type] = extractor.get_type().value
        if extractor.get_mode() == Mode.SIMPLE:
            vocab_dict[search_mod] = SearchMode.ORDERED_SPAN.value
            vocab_dict[analyze_strategy] = AnalyzeMode.SIMPLE.value
        elif extractor.get_mode() == Mode.UNORDERED:
            vocab_dict[search_mod] = SearchMode.SPAN.value
            vocab_dict[analyze_strategy] = AnalyzeMode.SIMPLE.value
        elif extractor.get_mode() == Mode.STEM:
            vocab_dict[search_mod] = SearchMode.ORDERED_SPAN.value
            vocab_dict[analyze_strategy] = AnalyzeMode.STEM.value
        elif extractor.get_mode() == Mode.UNORDERED_STEM:
            vocab_dict[search_mod] = SearchMode.SPAN.value
            vocab_dict[analyze_strategy] = AnalyzeMode.STEM.value
        elif extractor.get_mode() == Mode.FUZZY_UNORDERED_STEM:
            vocab_dict[search_mod] = SearchMode.FUZZY_SPAN.value
            vocab_dict[analyze_strategy] = AnalyzeMode.STEM.value
        if extractor.get_stop_word_list() is not None:
            vocab_dict[stop_word_list] = extractor.get_stop_word_list()
        if extractor.get_synonym_list() is not None:
            vocab_dict[synonym_l] = extractor.get_synonym_list()
        if extractor.get_validator() is not None:
            try:
                re.compile(extractor.get_validator())
                ok = True
            except re.error:
                ok = False
            if ok is False:
                raise QtArgumentError("Argument type error: Please write valid regular expression for validator.")
            else:
                vocab_dict[validator] = extractor.get_validator()
                vocab_dict[vocab_value_type] = extractor.get_vocab_value_type()
        self.temp_dictionary[search_vocabulary].append(vocab_dict)
        return self

    def get_extractor(self, dictionary: Dict) -> Model:
        """Get extractor"""

        if search_vocabulary in dictionary:
            for i in dictionary[search_vocabulary]:
                self.temp_dictionary[search_vocabulary].append(i)
        return self

    def clear(self) -> None:
        """Remove all temporary data"""

        self.temp_dictionary.pop(tag_files, None)
        self.temp_dictionary.pop(description, None)
        self.temp_dictionary[exclude_utt] = True
        self.temp_dictionary[num_workers] = 4
        self.temp_dictionary.pop(search_vocabulary, None)
        self.index = None

    def create(self) -> Model:
        """Creating a new model"""

        self.headers["Content-Type"] = "application/json"
        data = {}
        if not self.temp_dictionary[search_vocabulary]:
            raise QtModelError("Model error: Please add parameters using add_extractor function")
        if tag_files in self.temp_dictionary and search_vocabulary in self.temp_dictionary:
            data[tag_files] = self.temp_dictionary[tag_files]
        data[exclude_utt] = self.temp_dictionary[exclude_utt]
        data[num_workers] = self.temp_dictionary[num_workers]
        data[chunk] = self.temp_dictionary[chunk]
        if search_vocabulary in self.temp_dictionary:
            data[search_vocabulary] = self.temp_dictionary[search_vocabulary]
        if description in self.temp_dictionary:
            data[description] = self.temp_dictionary[description]
        res = connect("post", f"{self.url}search/new", self.headers, "data", json.dumps(data))
        self.id = res.json()['id']
        del self.headers['Content-Type']
        return self

    def set_returned_data(self, result: Dict) -> None:
        """Set returned data from server"""

        self.temp_dictionary[description] = result[description]
        self.temp_dictionary[exclude_utt] = result[exclude_utt]
        self.temp_dictionary[search_vocabulary] = result[search_vocabulary]
        self.temp_dictionary[tag_files] = result[tag_files]
        self.temp_dictionary[chunk] = result[chunk]
        self.temp_dictionary[num_workers] = result[num_workers]

    def fetch(self, model_id: str) -> Model:
        """ Fetch model where model_id is existing ID"""

        self.headers["Content-Type"] = "application/json"
        if not isinstance(model_id, str):
            raise QtArgumentError("Argument type error: String is expected as model_id")
        res = connect("get", f"{self.url}search/config/{model_id}", self.headers)
        result = res.json()
        del self.headers['Content-Type']
        self.id = model_id
        self.set_returned_data(result)
        return self

    def update(self, model_id: str, update_files: List) -> Model:
        """ Update model where model_id is existing ID"""

        self.headers["Content-Type"] = "application/json"
        if not isinstance(model_id, str):
            raise QtArgumentError("Argument type error: String is expected as model_id")
        if not isinstance(update_files, List):
            raise QtArgumentError("Argument type error: List is expected as update file")
        else:
            self.clear()
            data = {tag_files: update_files}
        res = connect("post", f"{self.url}search/update/{model_id}", self.headers, "data", json.dumps(data))
        result = res.json()
        del self.headers['Content-Type']
        self.id = model_id
        self.set_returned_data(result)
        return self

    def clone(self, model_id: str) -> Model:
        """ Update model where model_id is existing ID"""

        self.headers["Content-Type"] = "application/json"
        if not isinstance(model_id, str):
            raise QtArgumentError("Argument type error: String is expected as model_id")
        if tag_files not in self.temp_dictionary:
            raise QtModelError("Model: You have to add files using with_document method and Document class")
        data = {tag_files: self.temp_dictionary[tag_files]}
        res = connect("post", f"{self.url}search/new/{model_id}", self.headers, "data", json.dumps(data))
        result = res.json()
        self.id = model_id
        self.set_returned_data(result)
        del self.headers['Content-Type']
        return self

    def delete(self, model_id: str) -> bool:
        """Delete data container"""

        if not isinstance(model_id, str):
            raise QtArgumentError("Argument type error: String is expected as model_id")
        res = connect("delete", f"{self.url}search/{model_id}", self.headers)
        return res.ok

    def progress(self, model_id: str = None) -> Dict:
        """Show progress for submitted process"""

        url_path = "progress"
        if model_id is not None:
            if isinstance(model_id, str):
                url_path = f"{url_path}/{model_id}"
            else:
                raise QtArgumentError("Expected string")
        res = connect("get", f"{self.url}search/{url_path}", self.headers)
        return res.json()

    def wait_for_completion(self) -> None:
        """Wait for completion processes"""

        percentage = 0
        while percentage < 100:
            result = self.progress(self.id)
            percentage = result["progress"]
            print(f"Search progress {percentage}%")
            if percentage < 100:
                sleep(1)
        sleep(3)

    def search(self, model_id: str, param_from: int = 0, size: int = None, f1: int = None, f2: int = None) -> Dict:
        """Search full-text and faceted search"""

        if not isinstance(model_id, str):
            raise QtArgumentError("Argument type error: String is expected as model_id")
        if isinstance(param_from, int):
            parameters = [('from', param_from)]
        else:
            raise QtArgumentError("Argument type error: Integer is expected as parameter")
        if size is not None:
            if isinstance(size, int) and 0 < size <= 200:
                parameters.append(('size', size))
            else:
                raise QtArgumentError("Argument type error: Integer between 0 and 200 is expected as parameter")
        if f1 is not None and f2 is not None:
            parameters.append(('f', f1))
            parameters.append(('f', f2))
        elif f1 is None and f2 is None:
            pass
        else:
            raise QtArgumentError("Argument error: Query filters must be used in pairs")
        res = connect("get", f"{self.url}search/{model_id}", self.headers, "params", parameters)

        return res.json()

    def read(self) -> List:
        """ Fetch model where model_id is existing ID"""

        self.headers["Content-Type"] = "application/json"
        res = connect("get", f"{self.url}users/profile", self.headers)
        re_dict = res.json()
        del self.headers['Content-Type']
        if "settings" in re_dict:
            model_list = []
            for i in re_dict["settings"]:
                model = Model()
                if "files" in i:
                    document_list = []
                    for f in i["files"]:
                        document = Document()
                        document.set_id(f)
                        document_list.append(document)
                    model.with_documents(document_list)
                model.set_id(i["id"])
                model.set_description(i["title"])
                model.set_workers(i["numWorkers"])
                if "searchDictionaries" in i:
                    for ex in i["searchDictionaries"]:
                        model.get_extractor(ex)
                model_list.append(model)
        else:
            model_list = []
        return model_list



from __future__ import annotations
import json
from time import sleep
from typing import List, Dict
from qtcurate.qt import Qt
from qtcurate.exceptions import QtArgumentError, QtJobError
from qtcurate.connect import connect

description = "title"
files = "files"


class Job:
    def __init__(self):
        self.headers = {"X-API-Key": Qt.api_key}
        self.temp_dict = dict()
        self.url = Qt.url
        self.model_id = None
        self.id = None

    def __repr__(self):
        return f"{self.id}"

    def get_id(self) -> str:
        """Get job id"""

        return str(self.id)

    def set_description(self, title: str) -> Job:
        """Create title for Job"""

        if isinstance(title, str):
            self.temp_dict[description] = title
        else:
            raise QtArgumentError("Argument type error: String is expected as description")
        return self

    def with_documents(self, list_of_files: List) -> Job:
        """Create a list of existing files"""

        if isinstance(list_of_files, list):
            self.temp_dict[files] = list_of_files
        else:
            raise QtArgumentError("Argument type error: Expected list of file IDs")
        return self

    def with_model(self, model_id: str) -> Job:
        """Add model to Job"""

        if isinstance(model_id, str):
            self.model_id = model_id
        else:
            raise QtArgumentError("Argument type error: Expected string as model ID")
        return self

    def create(self) -> Dict:
        """Creating a new Job"""

        self.headers["Content-Type"] = "application/json"
        if description not in self.temp_dict:
            raise QtJobError("Job error: Description is mandatory. Please add description with set_description method")
        if files not in self.temp_dict:
            raise QtJobError("Job error: Please add list of documents with with_documents method ")

        res = connect("post", f"{self.url}search/new/{self.model_id}", self.headers,  "data", json.dumps(self.temp_dict))
        self.id = res.json()['id']
        del self.headers['Content-Type']
        return res.json()

    def fetch(self, job_id: str) -> Dict:
        """ Fetch job where job_id is existing ID"""

        self.headers["Content-Type"] = "application/json"
        if not isinstance(job_id, str):
            raise QtArgumentError("Argument type error: String is expected as job_id")
        res = connect("get", f"{self.url}search/config/{job_id}", self.headers)
        self.id = res.json()['id']
        del self.headers['Content-Type']
        return res.json()

    def delete(self, job_id: str) -> bool:
        """Delete data container"""

        if not isinstance(job_id, str):
            raise QtArgumentError("Argument type error: String is expected as job_id")
        res = connect("delete", f"{self.url}search/{job_id}", self.headers)
        return res.ok

    def progress(self, job_id: str = None) -> Dict:
        """Show progress for submitted for job"""

        url_path = "progress"
        if job_id is not None:
            if isinstance(job_id, str):
                url_path = f"{url_path}/{job_id}"
            else:
                raise QtArgumentError("Expected string")
        res = connect("get", f"{self.url}search/{url_path}", self.headers)
        return res.json()

    def wait_for_completion(self) -> None:
        """Wait for completion job"""

        percentage = 0
        while percentage < 100:
            result = self.progress(self.id)
            percentage = result["progress"]
            print(f"Search progress {percentage}%")
            if percentage < 100:
                sleep(1)
        sleep(3)



from qtcurate.qt import Qt
from smart_open import open
from qtcurate.exceptions import QtArgumentError
from qtcurate.connect import connect


class Document:

    def __init__(self):
        self.headers = {"X-API-Key": Qt.api_key}
        self.url = Qt.url
        self.id = None

    def __repr__(self):
        return f"{self.id}"

    def set_id(self, uuid: str) -> None:
        """Set document id"""

        if isinstance(uuid, str):
            self.id = uuid
        else:
            raise QtArgumentError("Argument type error: String is expected as uuid")

    def get_id(self) -> str:
        """Get document id"""

        return self.id

    def create(self, file: str) -> str:
        """Upload files for data mining"""

        if not isinstance(file, str):
            raise QtArgumentError("Argument type error: String is expected as file path")
        files = {'file': open(file, 'rb')}
        res = connect("post", f"{self.url}search/file", self.headers, "files", files)

        return res.json()["uuid"]




class QtConnectionError(Exception):
    pass


class QtRestApiError(Exception):
    """ Problem with authentification"""
    pass


class QtFileTypeError(Exception):
    """Invalid type of file"""
    pass



class QtArgumentError(Exception):
    pass


class QtVocabularyError(Exception):
    pass

class QtModelError(Exception):
    pass

class QtJobError(Exception):
    pass


from __future__ import annotations
from enum import Enum
from qtcurate.exceptions import QtArgumentError
from typing import List


class ChunkMode(Enum):
    NONE = "NONE"
    SENTENCE = "SENTENCE"
    PAGE = "PAGE"


class SearchMode(Enum):
    ORDERED_SPAN = "ORDERED_SPAN"
    FUZZY_ORDERED_SPAN = "FUZZY_ORDERED_SPAN"
    SPAN = "SPAN"
    FUZZY_SPAN = "FUZZY_SPAN"
    PARTIAL_SPAN = "PARTIAL_SPAN"
    PARTIAL_FUZZY_SPAN = "PARTIAL_FUZZY_SPAN"


class AnalyzeMode(Enum):
    EXACT = "EXACT"
    EXACT_CI = "EXACT_CI"
    WHITESPACE = "WHITESPACE"
    SIMPLE = "SIMPLE"
    STANDARD = "STANDARD"
    LETTER = "LETTER"
    STEM = "STEM"


class Type(Enum):
    LONG = "LONG"
    DATETIME = "DATETIME"
    DOUBLE = "DOUBLE"


class Mode(Enum):
    SIMPLE = "SIMPLE"
    UNORDERED = "UNORDERED"
    STEM = "STEM"
    UNORDERED_STEM = "UNORDERED_STEM"
    FUZZY_UNORDERED_STEM = "FUZZY_UNORDERED_STEM"


class Extractor:

    def __init__(self):
        self.validator = None
        self.mode = Mode.SIMPLE
        self.type = None
        self.vocab_id = None
        self.vocab_value_type = None
        self.between_values = None
        self.stop_word_list = None
        self.synonym_list = None
        self.data_type = None

    def __repr__(self):
        return f"{self.validator} {self.mode}, {self.type}, {self.vocab_id}, {self.vocab_value_type} {self.data_type}"

    def create_mode(self, analyze_mode: AnalyzeMode, search_mode: SearchMode) -> Extractor:
        """Create mode"""

        if analyze_mode.value == "SIMPLE" and search_mode.value == "SPAN":
            self.mode = Mode.UNORDERED
        elif analyze_mode.value == "STEM" and search_mode.value == "ORDERED_SPAN":
            self.mode = Mode.STEM
        elif analyze_mode.value == "STEM" and search_mode.value == "SPAN":
            self.mode = Mode.UNORDERED_STEM
        elif analyze_mode.value == "STEM" and search_mode.value == "FUZZY_SPAN":
            self.mode = Mode.FUZZY_UNORDERED_STEM
        else:
            self.mode = Mode.SIMPLE
        return self

    def set_mode(self, mode: Mode) -> None:
        """Set mode"""

        if isinstance(mode, Mode):
            self.mode = mode
        else:
            raise QtArgumentError("Argument type error: Mode is expected as mode")

    def get_mode(self) -> Mode:
        """Get mode"""

        return self.mode

    def set_vocabulary(self, vocabulary: str) -> Extractor:
        """Set vocabulary id"""

        if isinstance(vocabulary, str):
            self.vocab_id = vocabulary
        else:
            raise QtArgumentError("Argument type error: String is expected as vocab_id")
        return self

    def get_vocab_id(self) -> str:
        """Get vocabulary id"""

        return self.vocab_id

    def get_vocab_value_type(self) -> str:
        """Get vocabulary value type"""

        return self.vocab_value_type

    def set_between_values(self, between_values: str) -> Extractor:
        """Set between values"""

        if isinstance(between_values, str):
            self.between_values = between_values
        else:
            raise QtArgumentError("Argument type error: String is expected as between_values")
        return self

    def get_between_values(self) -> str:
        """Get between values"""

        return self.between_values

    def set_type(self, data_type: Type) -> Extractor:
        """Set data type"""

        if isinstance(data_type, Type):
            self.type = data_type
        else:
            raise QtArgumentError("Argument type error: String is expected as DataType")
        return self

    def get_type(self) -> Type:
        """Get data type"""

        return self.type

    def set_stop_word_list(self, stop_word_list: List) -> Extractor:
        """ Set stop word list"""

        if isinstance(stop_word_list, list):
            self.stop_word_list = stop_word_list
        else:
            raise QtArgumentError("Argument type error: List is expected as stop_word_list")
        return self

    def get_stop_word_list(self) -> List:
        """Get stop word list"""

        return self.stop_word_list

    def set_synonym_list(self, synonym_list: List) -> Extractor:
        """Set synonym list"""

        if isinstance(synonym_list, list):
            self.synonym_list = synonym_list
        else:
            raise QtArgumentError("Argument type error: List is expected as synonym_list")

        return self

    def get_synonym_list(self) -> List:
        """Get synonym list"""

        return self.synonym_list

    def set_validator(self, validator: str) -> Extractor:
        """Set validator"""

        if isinstance(validator, str):
            self.validator = validator
            self.vocab_value_type = "REGEX"
        else:
            raise QtArgumentError("Argument type error: String is expected as validator")
        return self

    def get_validator(self) -> str:
        """Get validator"""

        return self.validator
